#include "defs.h"
#include "image.h"

IMAGE *Image = NULL;

/*********************************************************
    PROC:  find_image()
    DOES:  finds image of the given name
**********************************************************/

IMAGE *find_image(char *name)
{
  IMAGE *image;

  for (image=Image; image; image=image->next) {
    if (!strcmp(name,image->name))
      return(image);
  }
  return((IMAGE *) NULL);
}

/*********************************************************
    PROC:  newimage()
    DOES:  creates a new image
**********************************************************/

IMAGE *newimage(char *name)
{
  IMAGE *i;

  i = (IMAGE *) malloc(sizeof(IMAGE));
  strcpy(i->name,name);
  i->data = (unsigned char *) NULL;
  i->next = Image;

  Image = i;
  return(i);
}

/************************************************************
	PROC:	read_ppm_image()
	DOES:	reads a ppm image, sets it up to be an OpenGL texture
************************************************************/

void read_ppm_image(IMAGE *image, FILE *fp) {
	unsigned char c1, c2;

	fscanf(fp,"%c%c",&c1, &c2);
	if (c1!='P' || c2!='6') {
		printf("Unable to open file for texture '%s': not a PPM (raw) file (c1=%c, c2=%c).\n",
			image->name,c1,c2);
		return;
	}
	int xsize, ysize, maxval;
	fscanf(fp,"%d%d%d",&xsize, &ysize, &maxval);
	printf("%s: %d x %d image\n",image->name,xsize,ysize);
	image->xsize = xsize;
	image->ysize = ysize;
	fgetc(fp);           // discard white-space character
	if (image->data)     // discard old image, if any
		free(image->data);
	int size = xsize*ysize*3;
	image->data = (unsigned char *) malloc(size);
	int count=0;
	int rv = fread(image->data, 1, size, fp);
	return;
	/*
	for (int n=0; n<size; n++) {
		int c=feof(fp);
		tprintf("feof = %d\n",c);
		if (c) {
			tprintf("early end of file: %d of %d bytes read, c=%d\n",count,size,c);
			break;
		}
		image->data[n] = f(unsigned char) fgetc(fp);
		count++;
	}
	*/
	count=0;
	while (1) {
		fgetc(fp);
		if (feof(fp)) {
			break;
		} else {
			count++;
		}
	}
	printf("%d bytes left in file\n",count);
}

///////////////////////////////////////////////////////////
//    PROC: img_save()
//    DOES: loads or saves viewing matrix
///////////////////////////////////////////////////////////

void img_save(char *fname)
{
  FILE *fp;
  const int maxVal=255;
  register int y;
  unsigned char *pixels;

  printf("Saving image %s\n",fname);
  fp = fopen(fname,"w");
  if (!fp) {
	printf("Unable to open file '%s'\n",fname);
	return;
  }
  fprintf(fp, "P6");
  fprintf(fp, "%d %d ", Win[0], Win[1]);
  fprintf(fp, "%d", maxVal);
  putc(13,fp);
  pixels = new unsigned char [3*Win[0]];

  y = 0;

  for ( y = Win[1]-1; y >=0; y-- ) {
	glReadPixels(0,y,Win[0],1,GL_RGB,GL_UNSIGNED_BYTE, (GLvoid *) pixels);
	for (int n=0; n<3*Win[0]; n++) {
		putc(pixels[n],fp);
	}
	//    fwrite(pixels, 3, Win[0], fp);
  }
  fclose(fp);
}

/*********************************************************
    PROC:  texload
    DOES:  reads in an rgb image
**********************************************************/

void texload(char *texname, char *fname, char *mode)
{ 
  FILE *fp; 
  IMAGE *image;
  GLuint texnum;

  printf("loading %s %s\n",texname,fname);
              /* open file */
  fp = fopen(fname,"r"); 
  if(!fp) {
	  printf("Unable to open input file\n");
  }
  image = find_image(texname);        /* check for existing image */
  if (!image) {
    image = newimage(texname);
  }
  strcpy(image->name,texname);
  image->clamp = !strcmp(mode,"clamp");
  read_ppm_image(image, fp);
  fclose(fp);
	// create an associated texture object,
    // which allows for efficient use of the texture
  glGenTextures(1, &texnum);
  image->num = texnum;
  glBindTexture(GL_TEXTURE_2D, texnum);
  printf("Texture %s has texture number %d\n",texname,texnum);
  tex_init(image);
}

//////////////////////////////////////////////////////////
//    PROC: tex_init()
//    DOES: asks OpenGL to build MIP-maps, enables textures
//////////////////////////////////////////////////////////

void tex_init(IMAGE *image)
{
      glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	  glTexImage2D(GL_TEXTURE_2D, 0, 3, image->xsize,
	  image->ysize, 0, GL_RGB, GL_UNSIGNED_BYTE,
        image->data);
	  if (image->clamp) {
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	  } else {
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	  }
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

////////////////////////////////////////////////////////////
//	PROC:	useTexture(char *name)
//	DOES:	does the texture binding for the given texture name
////////////////////////////////////////////////////////////

void useTexture(char *name)
{
	  // lookup the GL texture number 
	IMAGE *image;
	image = find_image(name);
	if (!image) {
		printf("texture %s not found\n",name);
		return;
	}

      // make it the current texture
//	printf("Binding texture %s\n",name);
    glBindTexture(GL_TEXTURE_2D, image->num);
	glEnable(GL_TEXTURE_2D);
}
