/************************************************************
	FILE:		user.cpp
	CONTAINS:	user functions
************************************************************/

#include "defs.h"
#include "image.h"

int Spin = 0;
int Angle = 0;
int SaveImage = 0;
int ImageNum = 0;

/////////////////////////////////////////////////////
// PROC:   idleCallBack()
// DOES: 
/////////////////////////////////////////////////////

void idleCallBack()
{
}

/////////////////////////////////////////////////////
//    PROC: glut_key_action()
//    DOES: handles key presses (ASCII)
/////////////////////////////////////////////////////

void glut_key_action(unsigned char key, int x, int y)
{
  if (key==' ') {
	  printf("Spin toggle to %d\n",Spin);
	  glutPostRedisplay();
	  Spin = !Spin;
  } else if (key=='s') {
	  SaveImage = !SaveImage;
  }
}

/************************************************************
	PROC:	user_init(void)
	DOES:	Initializes user-defined globals
************************************************************/

void user_init()
{
  glutDisplayFunc(display);
  glutReshapeFunc(myReshape);
  glutIdleFunc( idleCallBack );
  glClearColor(1.0,1.0,1.0,0.0);
  glutKeyboardFunc( glut_key_action );

     // read in the textures
     // note that the images must be of size 2^n by 2^n, i.e., 64x64, 128x128, etc.
  texload("tower","clocktower.ppm","clamp");
  texload("brick","brick.ppm","repeat");
}

////////////////////////////////////////////////////////////
//	PROC:	draw_square(void)
//	DOES:	draws a unit-sized square
////////////////////////////////////////////////////////////

void draw_square()
{
	glBegin(GL_POLYGON);

	glTexCoord2f(0,0);
	glVertex3f(0,0,0);
	
	glTexCoord2f(1,0);
	glVertex3f(1,0,0);
	
	glTexCoord2f(1,1);
	glVertex3f(1,1,0);
	
	glTexCoord2f(0,1);
	glVertex3f(0,1,0);
	
	glEnd();
}

////////////////////////////////////////////////////////////
//	PROC:	draw_world(void)
//	DOES:	draws the world
////////////////////////////////////////////////////////////

void draw_world()
{
		// regular bricks
	useTexture("brick");
	glColor3f(1,1,1);   
	draw_square();

		 // green bricks:  the colour multiplies the texture !
	glColor3f(0,1,0);  
	glTranslatef(0,-1,0);
	draw_square();

	    // plain green square
	glDisable(GL_TEXTURE_2D);
	glTranslatef(-1,0,0);
	glColor3f(0,1,0);
	draw_square();

	    // UBC clock tower
	useTexture("tower");
	glTranslatef(0,1,0);
	glColor3f(1,1,1);
	draw_square();
}

////////////////////////////////////////////////////////////
//	PROC:	display(void)
//	DOES:	draw the world
////////////////////////////////////////////////////////////

void display()
{
		  // clear screen
	    glClear((GLbitfield) (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
        glColor3f (1.0, 1.0, 1.0);

	      // clear all transformations
	    glMatrixMode(GL_PROJECTION);
	    glLoadIdentity();
	    glMatrixMode(GL_MODELVIEW);
	    glLoadIdentity();

		glRotatef(Angle,0.5,1,0);
		draw_world();
		glFlush();

		if (SaveImage) {    // save image ??
			char fname[100];
			ImageNum++;
			sprintf(fname,"img/img%03d.ppm",ImageNum);
			img_save(fname);
		}

		glutSwapBuffers();

		if (Spin) {         // if in spin mode, update angle, then request a redraw
			Angle += Spin;
			glutPostRedisplay();
		}
}

